#ifndef __JAVA_PARSER_CONSTANT_POOL_TAGS_TAGS_H
#define __JAVA_PARSER_CONSTANT_POOL_TAGS_TAGS_H

#include <SpecTypes.h>
#include <cstdint>
#include <cstdlib>
#include <string>

struct JavaClassFormat;

namespace ConstantPoolTags
{
enum class Tags : uint8_t
{
	Utf8 = 1,
	Integer = 3,
	Float,
	Long,
	Double,
	Class,
	String,
	Fieldref,
	Methodref,
	InterfaceMethodref,
	NameAndType,
	MethodHandle = 15,
	MethodType,
	Dynamic,
	InvokeDynamic,
	Module,
	Package
};


struct ConstantPoolInfo
{
	ConstantPoolTags::Tags tag;
};

struct ConstantPool
{
	ConstantPoolTags::Tags tag;
	ConstantPoolInfo info;
};

struct ConstantUtf8Info : ConstantPoolInfo
{
	ConstantPoolTags::Tags tag;
	u2 length;
	std::string bytes;
	// std::vector<unsigned char> bytes;
};

struct ConstantClassInfo : ConstantPoolInfo
{
	ConstantPoolTags::Tags tag;
	u2 name_index;
};

struct ConstantStringInfo : ConstantPoolInfo
{
	ConstantPoolTags::Tags tag;
	u2 string_index;
};

struct ConstantFieldrefInfo : ConstantPoolInfo
{
	ConstantPoolTags::Tags tag;
	u2 class_index;
	u2 name_and_type_index;
};

struct ConstantMethodrefInfo : ConstantPoolInfo
{
	ConstantPoolTags::Tags tag;
	u2 class_index;
	u2 name_and_type_index;
};

struct ConstantNameAndTypeInfo : ConstantPoolInfo
{
	ConstantPoolTags::Tags tag;
	u2 name_index;
	u2 descriptor_index;
};

void ConstantPoolParser(ConstantPoolTags::Tags t, JavaClassFormat &jc);
}

#endif